@@ -2,10 +2,44 @@
 # -*- coding: utf-8 -*-
 
 import itertools as it, operator as op, functools as ft
-from collections import Mapping
+from collections import Mapping, OrderedDict
 import os, sys
+import yaml, yaml.constructor
 
 
+class OrderedDictYAMLLoader(yaml.Loader):
+	'Based on: https://gist.github.com/844388'
+
+	def __init__(self, *args, **kwargs):
+		yaml.Loader.__init__(self, *args, **kwargs)
+		self.add_constructor(u'tag:yaml.org,2002:map', type(self).construct_yaml_map)
+		self.add_constructor(u'tag:yaml.org,2002:omap', type(self).construct_yaml_map)
+
+	def construct_yaml_map(self, node):
+		data = OrderedDict()
+		yield data
+		value = self.construct_mapping(node)
+		data.update(value)
+
+	def construct_mapping(self, node, deep=False):
+		if isinstance(node, yaml.MappingNode):
+			self.flatten_mapping(node)
+		else:
+			raise yaml.constructor.ConstructorError( None, None,
+				'expected a mapping node, but found {}'.format(node.id), node.start_mark )
+
+		mapping = OrderedDict()
+		for key_node, value_node in node.value:
+			key = self.construct_object(key_node, deep=deep)
+			try:
+				hash(key)
+			except TypeError, exc:
+				raise yaml.constructor.ConstructorError( 'while constructing a mapping',
+					node.start_mark, 'found unacceptable key ({})'.format(exc), key_node.start_mark )
+			value = self.construct_object(value_node, deep=deep)
+			mapping[key] = value
+		return mapping
+
 
 class AttrDict(dict):
 
@@ -24,7 +58,7 @@ class AttrDict(dict):
 	def from_yaml(cls, path, if_exists=False):
 		import yaml
 		if if_exists and not os.path.exists(path): return cls()
-		return cls(yaml.load(open(path)))
+		return cls(yaml.load(open(path), OrderedDictYAMLLoader))
 
 	def flatten(self, path=tuple()):
 		dst = list()
@@ -95,6 +129,15 @@ def main():
 		help='Explicitly disable specified datapoint sinks,'
 			' can be specified multiple times. Overrides --sink-enable.')
 
+	parser.add_argument('-p', '--processor-enable',
+		action='append', metavar='processor', default=list(),
+		help='Enable only the specified datapoint processors,'
+				' can be specified multiple times.')
+	parser.add_argument('-z', '--processor-disable',
+		action='append', metavar='processor', default=list(),
+		help='Explicitly disable specified datapoint processors,'
+			' can be specified multiple times. Overrides --processor-enable.')
+
 	parser.add_argument('-c', '--config',
 		action='append', metavar='path', default=list(),
 		help='Configuration files to process.'
@@ -104,8 +147,6 @@ def main():
 
 	parser.add_argument('-n', '--dry-run',
 		action='store_true', help='Do not actually send data.')
-	parser.add_argument('--debug-data', action='store_true',
-		help='Log datapoints that are (unless --dry-run is used) sent to carbon.')
 	parser.add_argument('--debug',
 		action='store_true', help='Verbose operation mode.')
 	optz = parser.parse_args()
@@ -129,7 +170,6 @@ def main():
 	log = logging.getLogger(__name__)
 
 	# Fill "auto-detected" blanks in the configuration, CLI overrides
-	if cfg.loop.prefix is None: cfg.loop.prefix = os.uname()[1].replace('.', '_')
 	try:
 		if optz.destination: cfg.sinks._default.host = optz.destination
 		cfg.sinks._default.host = cfg.sinks._default.host.rsplit(':', 1)
@@ -140,12 +180,12 @@ def main():
 	except KeyError: pass
 	if optz.interval: cfg.loop.interval = optz.interval
 	if optz.dry_run: cfg.debug.dry_run = optz.dry_run
-	if optz.debug_data: cfg.debug.dump = optz.debug_data
 
 	# Override "enabled" collector/sink parameters, based on CLI
 	ep_conf = dict()
 	for ep, enabled, disabled in\
 			[ ('collectors', optz.collector_enable, optz.collector_disable),
+				('processors', optz.processor_enable, optz.processor_disable),
 				('sinks', optz.sink_enable, optz.sink_disable) ]:
 		conf = cfg[ep]
 		conf_base = conf.pop('_default')
@@ -159,7 +199,7 @@ def main():
 			if name not in conf: conf[name] = dict()
 			conf[name]['enabled'] = False
 		if 'debug' not in conf_base: conf_base['debug'] = cfg.debug
-		ep_conf[ep] = conf_base, conf, set()
+		ep_conf[ep] = conf_base, conf, OrderedDict()
 
 	# Init global cfg for collectors/sinks' usage
 	from graphite_metrics import collectors, sinks, loops
@@ -168,29 +208,34 @@ def main():
 	# Init pluggable components
 	import pkg_resources
 
-	for ep_type in 'collector', 'sink':
+	for ep_type in 'collector', 'processor', 'sink':
 		ep_key = '{}s'.format(ep_type) # a bit of a hack
 		conf_base, conf, objects = ep_conf[ep_key]
-		for ep in pkg_resources\
-				.iter_entry_points('graphite_metrics.{}'.format(ep_key)):
-			if ep.name[0] == '_':
+		ep_dict = dict( (ep.name, ep.load()) for ep in
+			pkg_resources.iter_entry_points('graphite_metrics.{}'.format(ep_key)) )
+		eps = OrderedDict( (name, (ep_dict.pop(name), subconf))
+			for name, subconf in conf.viewitems() if name in ep_dict )
+		eps.update( (name, (module, conf_base))
+			for name, module in ep_dict.viewitems() )
+		for ep_name, (ep_module, subconf) in eps.viewitems():
+			if ep_name[0] == '_':
 				log.debug( 'Skipping {} enty point,'
-					' prefixed by underscore: {}'.format(ep_type, ep.name) )
-			subconf = conf.get(ep.name, conf_base)
+					' prefixed by underscore: {}'.format(ep_type, ep_name) )
 			# Fill "_default" collector parameters
 			for k,v in conf_base.viewitems():
 				if k not in subconf: subconf[k] = v
 			if subconf.get('enabled', True):
-				log.debug('Loading {}: {}'.format(ep_type, ep.name))
-				try: obj = getattr(ep.load(), ep_type)(subconf)
+				log.debug('Loading {}: {}'.format(ep_type, ep_name))
+				try: obj = getattr(ep_module, ep_type)(subconf)
 				except Exception as err:
-					log.exception('Failed to load/init collector ({}): {}'.format(ep.name, err))
+					log.exception('Failed to load/init collector ({}): {}'.format(ep_name, err))
 					subconf.enabled = False
-				if subconf.get('enabled', True): objects.add(obj)
+					obj = None
+				if subconf.get('enabled', True): objects[ep_name] = obj
 				else:
 					log.debug(( '{} {} (entry point: {})'
-						' was disabled after init' ).format(ep_type.title(), obj, ep.name))
-		if not objects:
+						' was disabled after init' ).format(ep_type.title(), obj, ep_name))
+		if ep_type != 'processor' and not objects:
 			log.fatal('No {}s were properly enabled/loaded, bailing out'.format(ep_type))
 			sys.exit(1)
 		log.debug('{}: {}'.format(ep_key.title(), objects))
@@ -201,11 +246,11 @@ def main():
 	if 'debug' not in conf: conf.debug = cfg.debug
 	loop = loop[cfg.loop.name].load().loop(conf)
 
-	collectors, sinks = it.imap( op.itemgetter(2),
-		op.itemgetter('collectors', 'sinks')(ep_conf) )
+	collectors, processors, sinks = it.imap( op.itemgetter(2),
+		op.itemgetter('collectors', 'processors', 'sinks')(ep_conf) )
 	log.debug(
-		'Starting main loop: {} ({} collectors, {} sinks)'\
-		.format(loop, len(collectors), len(sinks)) )
-	loop.start(collectors, sinks)
+		'Starting main loop: {} ({} collectors, {} processors, {} sinks)'\
+		.format(loop, len(collectors), len(processors), len(sinks)) )
+	loop.start(collectors, processors, sinks)
 
 if __name__ == '__main__': main()
